unit AJBDual;

interface

uses WinTypes,
    WinProcs,
    Messages,
    SysUtils,
    Classes,
    Controls,
    Forms,
    Graphics,
    Extctrls,
    Stdctrls;

type
    TDualList = class(TPanel)
    private
        FRemoveFromSource: Boolean;
        FSrcUnique: Boolean;

        function GetDstListBoxItems: TStrings;
        procedure SetDstListBoxItems(Value: TStrings);
        function GetDstSorted: Boolean;
        procedure SetDstSorted(Value: Boolean);
        function GetDstTitle: string;
        procedure SetDstTitle(Value: string);
        function GetSrcListBoxItems: TStrings;
        procedure SetSrcListBoxItems(Value: TStrings);
        function GetSrcSorted: Boolean;
        procedure SetSrcSorted(Value: Boolean);
        function GetSrcTitle: string;
        procedure SetSrcTitle(Value: string);
        procedure ListBoxDragOver(Sender, Source: TObject; X, Y: Integer;
            State: TDragState; var Accept: Boolean);
        procedure ListBoxDragDrop(Sender, Source: TObject; X, Y: Integer);
        procedure SetLabelFont(const Value: TFont);
        procedure SetListFont(const Value: TFont);
        function GetLabelFont: TFont;
        function GetListFont: TFont;
        procedure SetRemoveFromSource(const Value: Boolean);
        procedure SetSrcUnique(const Value: Boolean);

    protected
        DstLabel: TLabel;
        DstListBox: TListBox;
        SrcLabel: TLabel;
        SrcListBox: TListBox;
        addAll: TButton;
        addOne: TButton;
        removeAll: TButton;
        removeOne: TButton;

        procedure DoaddAllClick(Sender: TObject); virtual;
        procedure DoaddOneClick(Sender: TObject); virtual;
        procedure DoremoveAllClick(Sender: TObject); virtual;
        procedure DoremoveOneClick(Sender: TObject); virtual;
        procedure KeyPress(var Key: Char); override;
        procedure Paint; override;
        procedure Resize; override;

    public
        constructor Create(AOwner: TComponent); override;
        destructor Destroy; override;
        procedure SetupButtons;
        procedure SetUpLists; virtual;
    published
        property OnClick;
        property OnDblClick;
        property OnDragDrop;
        property OnEnter;
        property OnExit;
        property OnKeyDown;
        property OnKeyPress;
        property OnKeyUp;
        property OnMouseDown;
        property OnMouseMove;
        property OnMouseUp;
        property OnResize;
        property LabelFont: TFont read GetLabelFont write SetLabelFont;
        property ListFont: TFont read GetListFont write SetListFont;
        property RemoveFromSource: Boolean read FRemoveFromSource write
            SetRemoveFromSource;
        property DstListBoxItems: TStrings
            read GetDstListBoxItems write SetDstListBoxItems;
        { Destination list sorted? }
        property DstSorted: Boolean
            read GetDstSorted write SetDstSorted;
        { Title above destination list }
        property DstTitle: string read GetDstTitle write SetDstTitle;
        property Height default 209;
        property SrcListBoxItems: TStrings
            read GetSrcListBoxItems write SetSrcListBoxItems;
        { Source list Sorted? }
        property SrcSorted: Boolean
            read GetSrcSorted write SetSrcSorted;
        { Title above source list }
        property SrcTitle: string read GetSrcTitle write SetSrcTitle;
        property SrcUnique: Boolean read FSrcUnique write SetSrcUnique;
        property Width default 345;

    end;
    TConvertFunc = function(const ConvertData: string): string of object;
    ESpecialDual = class(Exception);
    TSpecialDual = class(TDualList)
    private
        FSourceKeyList: TStrings;
        procedure SetSourceKeyList(const Value: TStrings);

    protected

    public
        constructor Create(AOwner: TComponent); override;
        destructor Destroy; override;
        procedure LoadLists(const Source, Dest: TStrings; LoadFunction:
            TConvertFunc);
        procedure GetDestList(var Dest: TStrings; GetFunction: TConvertFunc);
        procedure GetDestListAsKeys(var Dest: TStrings);
    published
        property SourceKeyList: TStrings read FSourceKeyList write
            SetSourceKeyList;
    end;

procedure Register;

const
    msgNilKey = 'Key value is nil';
    msgKeyBounds = 'Key index is out of bounds';

implementation

procedure Register;
begin
     { Register TDualList with Extras as its
       default page on the Delphi component palette }
    RegisterComponents('Extras', [TDualList, TSpecialDual]);
end;

function TDualList.GetDstListBoxItems: TStrings;
begin
    Result := DstListBox.Items;
end;

procedure TDualList.SetDstListBoxItems(Value: TStrings);
begin
    DstListBox.Items.Assign(Value);
end;

function TDualList.GetDstSorted: Boolean;
begin
    Result := DstListBox.Sorted;
end;

procedure TDualList.SetDstSorted(Value: Boolean);
begin
    DstListBox.Sorted := Value;
end;

function TDualList.GetDstTitle: string;
begin
    Result := DstLabel.Caption;
end;

procedure TDualList.SetDstTitle(Value: string);
begin
    DstLabel.Caption := Value;
end;

function TDualList.GetSrcListBoxItems: TStrings;
begin
    Result := SrcListBox.Items;
end;

procedure TDualList.SetSrcListBoxItems(Value: TStrings);
var
    i, j, k, c: integer;

    procedure AddtoList;
    begin
        k := SrcListBox.Items.Add(Value[i]);
        SrcListBox.Items.Objects[k] := Pointer(c);
        Inc(c);
    end;
begin
    SrcListBox.Items.BeginUpdate;
    SrcListBox.Items.Clear;
    c := 0;
    for i := 0 to Value.Count - 1 do
    begin
        if FSrcUnique then
        begin
            j := SrcListBox.Items.IndexOf(Value[i]);
            if j < 0 then AddtoList;
        end
        else
            AddtoList;
    end;
    SrcListBox.Items.EndUpdate;
end;

function TDualList.GetSrcSorted: Boolean;
begin
    Result := SrcListBox.Sorted;
end;

procedure TDualList.SetSrcSorted(Value: Boolean);
begin
    SrcListBox.Sorted := Value;
end;

function TDualList.GetSrcTitle: string;
begin
    Result := SrcLabel.Caption;
end;

procedure TDualList.SetSrcTitle(Value: string);
begin
    SrcLabel.Caption := Value;
end;

procedure TDualList.DoaddAllClick(Sender: TObject);
begin
    srcListBox.Items.BeginUpdate;
    DstListBox.Items.BeginUpdate;
    dstListBox.Items.AddStrings(srcListBox.Items);
    if FRemoveFromSource then srcListBox.Items.Clear;
    srcListBox.Items.EndUpdate;
    DstListBox.Items.EndUpdate;
    SetUpButtons;
end;

procedure TDualList.DoaddOneClick(Sender: TObject);
var
    i: integer;
begin
    srcListBox.Items.BeginUpdate;
    DstListBox.Items.BeginUpdate;
    for i := 0 to srcListBox.Items.Count - 1 do
        if srcListBox.Selected[i] then
            dstListBox.Items.AddObject(srcListbox.Items[i],
                srcListbox.Items.Objects[i]);
    if FRemoveFromSource then
        for i := srcListBox.Items.Count - 1 downto 0 do
            if srcListBox.Selected[i] then
                srclistbox.Items.Delete(i);
    srcListBox.Items.EndUpdate;
    DstListBox.Items.EndUpdate;
    SetupButtons;
end;

procedure TDualList.DoremoveAllClick(Sender: TObject);
begin
    srcListBox.Items.BeginUpdate;
    DstListBox.Items.BeginUpdate;
    if FRemoveFromSource then srcListBox.Items.AddStrings(dstListBox.Items);
    dstListBox.Items.Clear;
    srcListBox.Items.EndUpdate;
    DstListBox.Items.EndUpdate;
    SetupButtons;
end;

procedure TDualList.DoremoveOneClick(Sender: TObject);
var
    i: integer;
begin
    srcListBox.Items.BeginUpdate;
    DstListBox.Items.BeginUpdate;
    for i := 0 to dstListBox.Items.Count - 1 do
        if (dstListBox.Selected[i]) and FRemoveFromSource then
            srcListBox.Items.AddObject(dstListbox.Items[i],
                dstListBox.Items.Objects[i]);
    for i := dstListBox.Items.Count - 1 downto 0 do
        if dstListBox.Selected[i] then
            dstlistbox.Items.Delete(i);
    srcListBox.Items.EndUpdate;
    DstListBox.Items.EndUpdate;
    SetupButtons;
end;

procedure TDualList.KeyPress(var Key: Char);
const
    TabKey = Char(VK_TAB);
    EnterKey = Char(VK_RETURN);
begin
    if Key = EnterKey then
    begin
        Key := #0;                           // processed
        if DstListBox.Focused then
            DoRemoveOneClick(nil)
        else
            DoAddOneClick(nil);
    end;
    inherited KeyPress(Key);
end;

constructor TDualList.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    DstLabel := TLabel.Create(Self);
    DstLabel.Parent := Self;
    with DstLabel do
    begin
        Left := 208;
        Top := 8;
        Width := 56;
        Height := 13;
        Anchors := [akRight, akTop];
        Caption := 'Destination:';
    end;
    DstListBox := TListBox.Create(Self);
    DstListBox.Parent := Self;
    with DstListBox do
    begin
        Left := 208;
        Top := 24;
        Width := 129;
        Height := 177;
        Anchors := [akTop, akBottom];
        DragMode := dmAutomatic;
        ItemHeight := 13;
        MultiSelect := True;
        TabOrder := 5;
        OnDragDrop := ListBoxDragDrop;
        OnDragOver := ListBoxDragOver;
        OnDblClick := DoRemoveOneClick;
        Tag := 2;
    end;
    SrcLabel := TLabel.Create(Self);
    SrcLabel.Parent := Self;
    with SrcLabel do
    begin
        Left := 8;
        Top := 8;
        Width := 37;
        Height := 13;
        Anchors := [akLeft, akTop];
        Caption := 'Source:';
    end;
    SrcListBox := TListBox.Create(Self);
    SrcListBox.Parent := Self;
    with SrcListBox do
    begin
        Left := 8;
        Top := 24;
        Width := 129;
        Height := 177;
        Anchors := [akLeft, akTop, akBottom];
        DragMode := dmAutomatic;
        ItemHeight := 13;
        MultiSelect := True;
        TabOrder := 0;
        OnDragDrop := ListBoxDragDrop;
        OnDragOver := ListBoxDragOver;
        OnDblClick := DoAddOneClick;
        Tag := 1;
    end;
    addAll := TButton.Create(Self);
    addAll.Parent := Self;
    with addAll do
    begin
        Left := 152;
        Top := 70;
        Width := 41;
        Height := 25;
        Anchors := [];
        Caption := '>>';
        TabOrder := 2;
        OnClick := DoaddAllClick;
    end;
    addOne := TButton.Create(Self);
    addOne.Parent := Self;
    with addOne do
    begin
        Left := 152;
        Top := 30;
        Width := 41;
        Height := 25;
        Anchors := [];
        Caption := '>';
        TabOrder := 1;
        OnClick := DoaddOneClick;
    end;
    removeAll := TButton.Create(Self);
    removeAll.Parent := Self;
    with removeAll do
    begin
        Left := 152;
        Top := 174;
        Width := 41;
        Height := 25;
        Anchors := [];
        Caption := '<<';
        TabOrder := 4;
        OnClick := DoremoveAllClick;
    end;
    removeOne := TButton.Create(Self);
    removeOne.Parent := Self;
    with removeOne do
    begin
        Left := 152;
        Top := 134;
        Width := 41;
        Height := 25;
        Anchors := [];
        Caption := '<';
        TabOrder := 3;
        OnClick := DoremoveOneClick;
    end;
    DstSorted := True;
    Height := 209;
    SrcSorted := True;
    Width := 345;
    Caption := '';                           // blank out caption;
    FRemoveFromSource := true;
    FSrcUnique := true;
    resize;
end;

destructor TDualList.Destroy;
begin
    DstLabel.Free;
    DstListBox.Free;
    SrcLabel.Free;
    SrcListBox.Free;
    addAll.Free;
    addOne.Free;
    removeAll.Free;
    removeOne.Free;
    inherited Destroy;
end;

procedure TDualList.Paint;
begin
    resize;
    inherited Paint;
end;

procedure TDualList.ListBoxDragDrop(Sender, Source: TObject; X,
    Y: Integer);
begin
    if (Sender is TListBox) and (Source is TListBox) then
    begin
        if (TListBox(Sender).Tag = 2) and
            (TListBox(Source).Tag = 1) then DoaddOneClick(nil);
        if (TListBox(Sender).Tag = 1) and
            (TListBox(Source).Tag = 2) then DoremoveOneClick(nil);
    end;
end;

procedure TDualList.ListBoxDragOver(Sender, Source: TObject; X, Y: Integer;
    State: TDragState; var Accept: Boolean);
begin
    if (Sender is TListBox) and (Source is TListBox) then
        if ((TListBox(Sender).Tag = 2) and
            (TListBox(Source).Tag = 1)) or
            ((TListBox(Sender).Tag = 1) and
            (TListBox(Source).Tag = 2)) then Accept := true;
end;

procedure TDualList.SetupButtons;
begin
    AddOne.Enabled := SrcListBox.Items.Count > 0;
    AddAll.Enabled := AddOne.Enabled;
    RemoveOne.Enabled := DstListBox.Items.Count > 0;
    RemoveAll.Enabled := RemoveOne.Enabled;
end;

procedure TDualList.SetUpLists;
var
    i, j: integer;

begin
    if (dstListBox.Items.Count > 0) and (SrcListBox.Items.Count > 0) then
    begin
        i := 0;
        while i < dstListBox.Items.Count do
        begin
            j := srcListBox.Items.IndexOf(dstListBox.Items[i]);
            if j > -1 then
            begin
                dstListBox.Items.Objects[i] := srcListBox.Items.Objects[j];
                if FRemoveFromSource then
                    srcListBox.Items.Delete(j);
            end
            else
                dstListBox.Items.Delete(i);
            inc(i)
        end;
    end;
end;

procedure TDualList.SetLabelFont(const Value: TFont);
begin
    SrcLabel.Font.Assign(Value);
    dstLabel.Font.Assign(Value);
    Refresh;
end;

procedure TDualList.SetListFont(const Value: TFont);
begin
    SrcListBox.Font.Assign(Value);
    DstListBox.Font.Assign(Value);
    ReFresh;
end;

function TDualList.GetLabelFont: TFont;
begin
    Result := SrcLabel.Font;
end;

function TDualList.GetListFont: TFont;
begin
    Result := SrcListBox.Font;
end;

procedure TDualList.Resize;
var
    ListWidth, hgap, lButton, vgap, vtop: integer;
begin
    inherited;
    hgap := SrcListBox.Left;
    lbutton := (Width - addone.Width) div 2;
    ListWidth := lButton - 2 * hgap;
    SrcListBox.Width := ListWidth;
    SrcListBox.Height := Height - SrcListBox.Top - hgap;
    DstListBox.Height := SrcListBox.Height;
    vgap := SrcListBox.Height div 14;
    vtop := SrcListBox.Top;

    addone.Left := lbutton;
    addone.Top := vtop + vgap;
    addall.Left := lbutton;
    addall.Top := vtop + 4 * vgap;
    removeone.Left := lbutton;
    removeone.Top := vtop + 8 * vgap;
    removeall.Left := lbutton;
    removeall.Top := vtop + 11 * vgap;

    DstListBox.Left := addone.Width + addone.Left + hgap;
    DstListBox.Width := ListWidth;
    DstLabel.Left := DstListBox.Left;
end;

procedure TDualList.SetRemoveFromSource(const Value: Boolean);
begin
    FRemoveFromSource := Value;
end;

procedure TDualList.SetSrcUnique(const Value: Boolean);
begin
    FSrcUnique := Value;
end;

{ TSpecialDual }

constructor TSpecialDual.Create(AOwner: TComponent);
begin
    inherited;
    FSourceKeyList := TStringList.Create;
end;

destructor TSpecialDual.Destroy;
begin
    FSourceKeyList.Free;
    inherited;
end;

procedure TSpecialDual.GetDestList(var Dest: TStrings;
    GetFunction: TConvertFunc);
var
    i: integer;
begin
    Dest.Clear;
    if Assigned(GetFunction) then
    begin
        for i := 0 to DstListBox.Items.Count - 1 do
            Dest.Add(GetFunction(DstListBox.Items[i]));
    end
    else
        Dest.Assign(DstListBox.Items);
end;

procedure TSpecialDual.GetDestListAsKeys(var Dest: TStrings);
var
    i, c, j: integer;
begin
    Dest.Clear;
    c := FSourceKeyList.Count;
    if c > 0 then
    begin
        for i := 0 to DstListBox.Items.Count - 1 do
        begin
            if DstListBox.Items.Objects[i] = nil then
                raise
                    ESpecialDual.Create(msgNilKey);
            j := Integer(DstListBox.Items.Objects[i]);
            if j < c then
                Dest.Add(FSourceKeyList[j])
            else
                raise ESpecialDual.Create(msgKeyBounds);
        end;
    end;
end;

procedure TSpecialDual.LoadLists(const Source, Dest: TStrings;
    LoadFunction: TConvertFunc);
var
    i, j, c: integer;
    s: string;
begin
    if Assigned(LoadFunction) then
    begin
        SrcListBox.Items.BeginUpdate;
        DstListBox.Items.BeginUpdate;
        SrcListBox.Items.Clear;
        c := 0;                              // index into key list
        for i := 0 to Source.Count - 1 do
        begin
            s := LoadFunction(Source[i]);
            if FSrcUnique then
            begin
                j := SrcListBox.Items.IndexOf(s);
                if j < 0 then
                begin
                    SrcListBox.Items.AddObject(s, Pointer(c));
                    Inc(c);
                end;
            end
            else
            begin
                SrcListBox.Items.AddObject(s, Pointer(c));
                Inc(c);
            end;
        end;
        DstListBox.Items.Clear;
        for i := 0 to Dest.Count - 1 do
        begin
            DstListBox.Items.Add(LoadFunction(Dest[i]));
        end;
    end
    else
    begin
        SrcListBox.Items.Clear;
        for i := 0 to Source.Count - 1 do
            SrcListBox.Items.AddObject(Source[i], Pointer(i));
        DstListBox.Items.Assign(Dest);
    end;
    SrcListBox.Items.EndUpdate;
    DstListBox.Items.EndUpdate;
    SetUpLists;
end;

procedure TSpecialDual.SetSourceKeyList(const Value: TStrings);
begin
    FSourceKeyList.Assign(Value);
end;

end.

