unit enableform;

interface

uses
    Windows,
    Messages,
    SysUtils,
    Classes,
    Graphics,
    Controls,
    Forms,
    Dialogs,
    StdCtrls,
    Buttons,
    DsgnIntf,
    ExtCtrls,
    contnrs,
    AJBDual;

type
    TT_EnableForm = class(TForm)
        DualList: TDualList;
        BitBtn1: TBitBtn;
        BitBtn2: TBitBtn;
    private
    { Private declarations }

    public
    { Public declarations }

    end;

    TGroupListEditor = class(TComponentEditor)
    private
        Flist, FUnallocated, AllocList: TStringlist;
        FAllowed, FForbidden: TStringList;
    public
        function GetVerb(Index: integer): string; override;
        function GetVerbCount: integer; override;
        procedure executeVerb(Index: integer); override;
        procedure ScanForm(list: TObjectlist);
        procedure AddComponentsToList(comp: TComponent);
        function IsLike(Value: TComponent; Name: string): Boolean;  //returns true if TControl is in the Ancestor
        function GetComponentFromName(Name: string): TComponent;
        constructor Create(Component: TComponent; Designer: IFormDesigner);
            override;
        destructor Destroy; override;
        procedure PutItemsOnList(list: TObjectList; S_List: TStrings);
    end;

{var
  Form2: TEnableForm;}

implementation

uses enablegroup;

{$R *.DFM}

procedure TGroupListEditor.executeVerb(Index: integer);
var
    MyForm: TT_EnableForm;
    GroupList: TObjectList;
    grp: TCustomEnableGroup;
begin
    MyForm := TT_EnableForm.Create(nil);
    grp := TCustomEnableGroup(Component);
    GroupList := grp.GroupList;
    FAllowed := grp.AllowedTypes;
    FForbidden := grp.ForbiddenTypes;
    ScanForm(GroupList);
    MyForm.DualList.SrcListBoxItems.Assign(FUnAllocated);
    MyForm.DualList.DstListBoxItems.Assign(AllocList);
    MyForm.DualList.SetupButtons;

    if (MyForm.ShowModal = mrOK) then
    begin
        PutItemsOnList(GroupList, MyForm.DualList.DstListBoxItems);
        grp.UpdateList;
    end;
    MyForm.Free;
end;

function TGroupListEditor.GetVerb(Index: integer): string;
begin
    Result := 'Edit Group details';
end;

function TGroupListEditor.GetVerbCount: integer;
begin
    Result := 1;
end;

procedure TGroupListEditor.AddComponentsToList(comp: TComponent);
var
    i, j: integer;
    res: Boolean;
begin
    if comp.ComponentCount > 0 then
        for i := 0 to comp.ComponentCount - 1 do
            AddComponentsToList(comp.Components[i]);
    res := False;
    for j := 0 to FAllowed.Count - 1 do
        if IsLike(comp, FAllowed[j]) then res := True;
    for j := 0 to FForbidden.Count - 1 do
        if IsLike(comp, FForbidden[j]) then res := False;
    if (res) and (comp.Name <> '') then
        Flist.AddObject(comp.Name, comp);
end;

function TGroupListEditor.GetComponentFromName(Name: string): TComponent;
var
    i: integer;
begin
    Result := nil;
    if Flist.Count > 0 then
    begin
        i := 0;
        while (i < Flist.Count) and (FList[i] <> Name) do
            Inc(i);
        if (i < FList.Count) then Result := TComponent(Flist.Objects[i]);
    end;
end;

function TGroupListEditor.IsLike(Value: TComponent; Name: string): Boolean;
var
    ClassRef: TClass;
begin
    ClassRef := Value.ClassType;
    while (ClassRef <> nil) and (ClassRef.ClassName <> Name) do
    begin
        ClassRef := ClassRef.ClassParent;
    end;
    Result := (ClassRef <> nil);
end;

procedure TGroupListEditor.ScanForm(list: TObjectlist);
var
    comp: TComponent;
    i: integer;
begin
    if (not Assigned(Flist)) or (not Assigned(list)) then
        Exit;

    Flist.Clear;
    FUnallocated.Clear;
    AllocList.Clear;
    comp := Component.Owner;
    AddComponentsToList(comp);
    for i := 0 to Flist.Count - 1 do
        if (list.IndexOf(Flist.Objects[i]) < 0) then
            FUnallocated.Add(Flist[i])
        else
            Alloclist.Add(Flist[i]);
end;

constructor TGroupListEditor.Create(Component: TComponent;
    Designer: IFormDesigner);
begin
    inherited;
    Flist := TStringList.Create;
    FUnallocated := TStringList.Create;
    AllocList := TStringList.Create;
end;

destructor TGroupListEditor.Destroy;
begin
    Flist.Free;
    FUnallocated.Free;
    AllocList.Free;
    inherited;
end;

procedure TGroupListEditor.PutItemsOnList(list: TObjectList; S_List: TStrings);
var
    i: integer;
    comp: TComponent;

begin
    if (not Assigned(list)) or (not Assigned(S_list)) then Exit;
    list.Clear;
    if S_list.Count > 0 then
        for i := 0 to S_list.Count - 1 do
        begin
            comp := GetComponentFromName(S_List[i]);
            if comp <> nil then list.Add(comp);
        end;
end;

end.

